"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importStar(require("react"));
var react_router_dom_1 = require("react-router-dom");
var getRoutes = function (r) {
    return r.keys().map(function (route) {
        var path = route
            .substr(1)
            .replace(/\/src\/pages|index|\.(js|jsx|ts|tsx)$/g, '')
            .replace(/\[\.{3}.+\]/, '*')
            .replace(/\[(.+)\]/, ':$1');
        var Element = react_1["default"].lazy(function () { return r(route); });
        return { path: path, element: react_1["default"].createElement(Element, null) };
    });
};
var getSyncRoutes = function (r) {
    return r.keys().map(function (route) {
        var path = route.substr(1).replace(/\/src\/pages|index|\.(js|jsx|ts|tsx)$/g, '');
        var Element = r(route)["default"];
        return { path: path, element: react_1["default"].createElement(Element, null) };
    });
};
function getMetaData(key, routes) {
    var data = {};
    routes
        .filter(function (route) { return route.path.includes(key); })
        .forEach(function (route) {
        var splitied = route.path.split('/');
        var indexoF = splitied.indexOf(key);
        var newElement = key === 'layout' ? (0, react_1.cloneElement)(route.element, { children: react_1["default"].createElement(react_router_dom_1.Outlet, null) }) : route.element;
        data[splitied[indexoF - 1]] = {
            el: newElement,
            pathLength: splitied.length
        };
    });
    return data;
}
function nestedRoutes(mainRoutes, layoutMetaRoutes, errorMetaRoutes, loadingMetatRoutes) {
    var nested = [];
    mainRoutes.forEach(function (route) {
        var currentNode = nested;
        var splittedPath = route.path.split('/');
        splittedPath.forEach(function (part, i) {
            var _a, _b;
            var isChild = i === splittedPath.length - 1;
            var loadingMetatRoutesArray = Object.values(loadingMetatRoutes);
            var Loading = (_a = loadingMetatRoutesArray
                .sort(function (a, b) { return b.pathLength - a.pathLength; })
                .find(function (e) { return e.pathLength <= splittedPath.length; })) === null || _a === void 0 ? void 0 : _a.el;
            var childNode = currentNode === null || currentNode === void 0 ? void 0 : currentNode.find(function (node) { return node.path === part; });
            if (!childNode) {
                childNode = {
                    path: part === '404' ? '*' : part,
                    fullPath: route.path,
                    element: isChild ? (react_1["default"].createElement(react_1["default"].Suspense, { fallback: Loading }, route.element)) : layoutMetaRoutes[part] ? (layoutMetaRoutes[part].el) : (''),
                    children: isChild ? undefined : [],
                    errorElement: (_b = Object.values(errorMetaRoutes).find(function (e) { return e.pathLength === splittedPath.length; })) === null || _b === void 0 ? void 0 : _b.el
                };
                if (i === 0) {
                    nested.push(childNode);
                }
                else {
                    currentNode.push(childNode);
                }
            }
            if (childNode.children) {
                currentNode = childNode.children;
            }
        });
    });
    return nested;
}
function collectedRoutes() {
    var requireAll = require.context('./../../../../../src/pages', true, /^\.\/(?!pages\/)(?!.*(?:loading|layout))[\w\/\[\]\-_]+\.(tsx|jsx|js|ts)?$/, 'lazy');
    var loadingAll = require.context('./../../../../../src/pages', true, /^\.\/(?!pages\/).*loading.*\.(js|jsx|ts|tsx)$/);
    var layoutAll = require.context('./../../../../../src/pages', true, /^\.\/(?!pages\/).*layout.*\.(js|jsx|ts|tsx)$/);
    var routes = getRoutes(requireAll);
    var loadingRoutes = getSyncRoutes(loadingAll);
    var layoutRoutes = getSyncRoutes(layoutAll);
    var loadingMetatRoutes = getMetaData('loading', loadingRoutes);
    var layoutMetaRoutes = getMetaData('layout', layoutRoutes);
    var errorMetaRoutes = getMetaData('500', routes);
    return nestedRoutes(routes, layoutMetaRoutes, errorMetaRoutes, loadingMetatRoutes);
}
exports["default"] = collectedRoutes;
